"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApolloSpreadSheet = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const GridWrapper_1 = tslib_1.__importDefault(require("./gridWrapper/GridWrapper"));
const ColumnGrid_1 = tslib_1.__importDefault(require("./columnGrid/ColumnGrid"));
const keyboard_1 = require("./keyboard");
const types_1 = require("./types");
const mergeCells_1 = require("./mergeCells");
const columnGrid_1 = require("./columnGrid");
const data_1 = require("./data");
const rowSelection_1 = require("./rowSelection");
const material_1 = require("@mui/material");
const editorManager_1 = require("./editorManager");
const react_dom_1 = require("react-dom");
const gridContainer_1 = require("./gridContainer");
const api_1 = require("./api");
const useEvents_1 = require("./events/useEvents");
const useSort_1 = require("./sort/useSort");
const logger_1 = require("./logger");
const helpers_1 = require("./helpers");
const nestedRows_1 = require("./nestedRows");
const theme_1 = require("./theme");
const useNestedColumns_1 = require("./nestedColumns/useNestedColumns");
/**
 * @todo I need to ensure this is only loaded in development, seems that with
 * process.env.NODE_ENV its not working when shipped and in Next.js applications
 * it throws a Loadable component error
 */
/*
if (process.env.NODE_ENV === 'development') {
  // eslint-disable-next-line global-require
  const whyDidYouRender = require('@welldone-software/why-did-you-render')
  whyDidYouRender(React, {
    trackAllPureComponents: false,
    trackHooks: true,
  })
}
*/
const rootDivStyle = { height: '100%', width: '100%' };
exports.ApolloSpreadSheet = (0, react_1.forwardRef)((props, componentRef) => {
    var _a, _b, _c, _d, _e, _f, _g;
    const logger = (0, logger_1.useLogger)('ApolloSpreadSheet');
    const minColumnWidth = (_a = props.minColumnWidth) !== null && _a !== void 0 ? _a : 30;
    const [gridFocused, setGridFocused] = (0, react_1.useState)(true);
    const defaultApiRef = (0, api_1.useApiRef)();
    const apiRef = react_1.default.useMemo(() => (!props.apiRef ? defaultApiRef : props.apiRef), [props.apiRef, defaultApiRef]);
    const rootContainerRef = (0, react_1.useRef)(null);
    const forkedRef = (0, material_1.useForkRef)(rootContainerRef, componentRef);
    const initialised = (0, api_1.useApiFactory)(rootContainerRef, apiRef, (_b = props.selection) === null || _b === void 0 ? void 0 : _b.key);
    const nestedRowsEnabled = (_c = props.nestedRows) !== null && _c !== void 0 ? _c : false;
    const nestedColumnsEnabled = (_d = props.nestedColumns) !== null && _d !== void 0 ? _d : false;
    (0, useEvents_1.useEvents)(rootContainerRef, apiRef);
    const theme = (0, theme_1.useTheme)({ apiRef, options: props.theme });
    const { gridHeaders, columns } = (0, columnGrid_1.useHeaders)({
        columns: props.columns,
        nestedHeaders: props.nestedHeaders,
        minColumnWidth,
        apiRef,
        initialised,
        selection: props.selection,
        nestedColumnsEnabled,
    });
    const { mergedPositions, mergedCells, isMerged } = (0, mergeCells_1.useMergeCells)({
        mergeCells: (_e = props.mergeCells) !== null && _e !== void 0 ? _e : [],
        rowCount: props.rows.length,
        columnCount: columns.length,
        apiRef,
        initialised,
    });
    const { cells, rows } = (0, data_1.useData)({
        rows: props.rows,
        selection: props.selection,
        apiRef,
        initialised,
        nestedRowsEnabled,
    });
    (0, nestedRows_1.useNestedRows)({
        apiRef,
        initialised,
        enabled: nestedRowsEnabled,
        defaultExpandedIds: props.defaultExpandedIds,
    });
    (0, useNestedColumns_1.useNestedColumns)({
        apiRef,
        initialised,
        enabled: nestedColumnsEnabled,
        defaultExpandedColumnsIds: props.defaultExpandedColumnsIds,
    });
    const sort = (0, useSort_1.useSort)(apiRef);
    const { coords, wasClicked } = (0, keyboard_1.useKeyboard)({
        defaultCoords: (_f = props.defaultCoords) !== null && _f !== void 0 ? _f : {
            rowIndex: 0,
            colIndex: 0,
        },
        suppressControls: props.suppressNavigation || !gridFocused,
        onCellChange: props.onCellChange,
        onCreateRow: props.onCreateRow,
        apiRef,
        initialised,
    });
    (0, rowSelection_1.useRowSelection)(apiRef, initialised, props.selection);
    const editorNode = (0, editorManager_1.useEditorManager)({
        onCellChange: props.onCellChange,
        apiRef,
        initialised,
    });
    //Ensure the root container is focused
    (0, react_1.useEffect)(() => {
        var _a;
        (_a = rootContainerRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    }, []);
    const focus = (0, react_1.useCallback)(() => {
        var _a, _b;
        logger.debug('focusing the grid with default coordinates ');
        if (!rootContainerRef.current) {
            logger.warn('Root container reference not defined yet');
        }
        (_a = rootContainerRef.current) === null || _a === void 0 ? void 0 : _a.focus();
        setGridFocused(true);
        apiRef.current.selectCell((_b = props.defaultCoords) !== null && _b !== void 0 ? _b : { colIndex: 0, rowIndex: 0 });
    }, [apiRef, logger, props.defaultCoords]);
    const clearFocus = (0, react_1.useCallback)(() => {
        var _a;
        if (gridFocused) {
            logger.debug('clearing the grid focus and selecting negative coordinates');
            setGridFocused(false);
            (_a = rootContainerRef.current) === null || _a === void 0 ? void 0 : _a.blur();
            apiRef.current.selectCell({
                rowIndex: -1,
                colIndex: -1,
            });
        }
    }, [apiRef, gridFocused, logger]);
    const onClickAway = (0, react_1.useCallback)((event) => {
        if (!gridFocused) {
            return;
        }
        //Handle callback style
        if ((0, helpers_1.isFunctionType)(props.outsideClickDeselects)) {
            const shouldClearFocus = props.outsideClickDeselects(event.target);
            if (shouldClearFocus) {
                logger.debug(`Grid click away detected. Callback returned false on target element id: ${event.target.id}`);
                return clearFocus();
            }
        }
        else if (props.outsideClickDeselects) {
            logger.debug('Grid outsideClickDeselects is passed as `true` value');
            clearFocus();
        }
    }, [gridFocused, props, logger, clearFocus]);
    // Detect if any element is clicked again to enable focus
    const onCellMouseHandler = (0, react_1.useCallback)(() => {
        if (!gridFocused) {
            logger.debug('Grid focus restored.');
            setGridFocused(true);
        }
    }, [gridFocused, logger]);
    // Clear every listener when component unmounts/apiRef changes
    (0, react_1.useEffect)(() => {
        const api = apiRef.current;
        return () => {
            api.removeAllListeners();
        };
    }, [apiRef]);
    (0, api_1.useApiEventHandler)(apiRef, api_1.CELL_CLICK, onCellMouseHandler);
    (0, api_1.useApiEventHandler)(apiRef, api_1.CELL_DOUBLE_CLICK, onCellMouseHandler);
    const rootApiMethods = { clearFocus, focus, isFocused: gridFocused };
    (0, api_1.useApiExtends)(apiRef, rootApiMethods, 'CoreApi');
    const nestedRowsProps = {
        nestedRows: nestedRowsEnabled,
        nestedRowMargin: props.nestedRowMargin,
        defaultExpandedIds: props.defaultExpandedIds,
        iconRenderer: props.iconRenderer,
    };
    const nestedColumnsProps = {
        nestedColumns: nestedColumnsEnabled,
        nestedColumnMargin: props.nestedColumnMargin,
        defaultExpandedColumnsIds: props.defaultExpandedColumnsIds,
        iconRenderer: props.iconRenderer,
    };
    const onScrollTable = (0, react_1.useCallback)((e) => {
        var _a;
        if (props === null || props === void 0 ? void 0 : props.id) {
            const header = document.getElementById(`header-${props === null || props === void 0 ? void 0 : props.id}`);
            if (!header) {
                return console.warn('The element does not exist on page');
            }
            header.scrollTo(e.scrollLeft, header.scrollTop);
        }
        if ((props === null || props === void 0 ? void 0 : props.connectToIds) && ((_a = props === null || props === void 0 ? void 0 : props.connectToIds) === null || _a === void 0 ? void 0 : _a.length) > 0) {
            props.connectToIds.map((element) => {
                const elementCore = document.getElementById(`core-${element}`);
                if (!elementCore) {
                    return console.warn(`The element ${element} does not exist on page`);
                }
                elementCore.scrollTo(e.scrollLeft, e.scrollTop);
                return elementCore;
            });
        }
    }, [props]);
    const onScrollHeader = (0, react_1.useCallback)((e) => {
        if (props === null || props === void 0 ? void 0 : props.id) {
            const elementCore = document.getElementById(`core-${props === null || props === void 0 ? void 0 : props.id}`);
            if (!elementCore) {
                return console.warn('The element does not exist on page');
            }
            elementCore.scrollTo(e.scrollLeft, elementCore.scrollTop);
        }
    }, [props === null || props === void 0 ? void 0 : props.id]);
    react_1.default.useEffect(() => {
        if ((props === null || props === void 0 ? void 0 : props.getSelectedCoords) && wasClicked && coords) {
            props === null || props === void 0 ? void 0 : props.getSelectedCoords(coords);
        }
    }, [coords, props, wasClicked]);
    return (react_1.default.createElement(material_1.ClickAwayListener, { onClickAway: onClickAway },
        react_1.default.createElement("div", { style: rootDivStyle, id: "root-apollo", ref: forkedRef },
            react_1.default.createElement(gridContainer_1.GridContainer, { columns: columns, minColumnWidth: minColumnWidth, stretchMode: (_g = props.stretchMode) !== null && _g !== void 0 ? _g : types_1.StretchMode.All, containerClassName: props.containerClassName, apiRef: apiRef, height: props.height, width: props.width }, ({ scrollLeft, onScroll, getColumnWidth, width, height }) => {
                var _a, _b, _c;
                return initialised && (react_1.default.createElement(material_1.Box, { height: height, width: width, id: "apollo-grids", className: props.className },
                    react_1.default.createElement(ColumnGrid_1.default, Object.assign({}, props, { data: gridHeaders, coords: coords, columns: columns, width: width, minColumnWidth: minColumnWidth, getColumnWidth: getColumnWidth, minRowHeight: (_a = props.minColumnHeight) !== null && _a !== void 0 ? _a : 50, scrollLeft: scrollLeft, onScrollHeader: (props === null || props === void 0 ? void 0 : props.id) ? onScrollHeader : () => null, onColumnCollapse: props === null || props === void 0 ? void 0 : props.onColumnCollapseChange, apiRef: apiRef, sort: sort, nestedRowsEnabled: nestedRowsEnabled, nestedColumnsProps: nestedColumnsProps, theme: theme, headerId: `header-${props === null || props === void 0 ? void 0 : props.id}` })),
                    react_1.default.createElement(GridWrapper_1.default, Object.assign({}, props, { rows: rows, data: cells, coords: coords, minColumnWidth: minColumnWidth, width: width, getColumnWidth: getColumnWidth, minRowHeight: (_b = props.minRowHeight) !== null && _b !== void 0 ? _b : 50, scrollLeft: scrollLeft, onScroll: (props === null || props === void 0 ? void 0 : props.id) ? onScrollTable : () => null, height: height, columnCount: columns.length, columns: columns, stretchMode: (_c = props.stretchMode) !== null && _c !== void 0 ? _c : types_1.StretchMode.All, apiRef: apiRef, mergeCells: mergedCells, mergedPositions: mergedPositions, isMerged: isMerged, onRowCollapse: props === null || props === void 0 ? void 0 : props.onRowCollapseChange, displayCollapseIcon: props === null || props === void 0 ? void 0 : props.displayCollapseIcon, nestedRowsProps: nestedRowsProps, theme: theme, coreId: `core-${props === null || props === void 0 ? void 0 : props.id}` }))));
            }),
            editorNode && (0, react_dom_1.createPortal)(editorNode, document.body))));
});
// @ts-ignore
//ApolloSpreadSheet.whyDidYouRender = true
exports.default = exports.ApolloSpreadSheet;
