"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useHeaders = void 0;
const react_1 = require("react");
const rowSelection_1 = require("../../rowSelection");
const api_1 = require("../../api");
const logger_1 = require("../../logger");
const gridWrapper_1 = require("../../gridWrapper");
const createSelectionColumn_1 = require("../../rowSelection/createSelectionColumn");
const flatExpandedColumns_1 = require("../../data/flatExpandedColumns");
/**
 * useHeaders manage the column configurations, parses
 * and validates but also expose util methods necessary for the creation of the headers
 * @todo Unit tests
 * @param headers
 * @param nestedHeaders
 */
function useHeaders({ columns, nestedHeaders, initialised, apiRef, selection, nestedColumnsEnabled, }) {
    const logger = (0, logger_1.useLogger)('useHeaders');
    const columnsRef = (0, react_1.useRef)(columns);
    const nestedHeadersRef = (0, react_1.useRef)(nestedHeaders);
    const [gridHeaders, setGridHeaders] = (0, react_1.useState)([[]]);
    const originalColumnsRef = (0, react_1.useRef)(columns);
    const createGridHeaders = (0, react_1.useCallback)(({ columns: paramColumns, nestedHeaders: paramNestedHeaders, }) => {
        logger.debug('Creating grid headers.');
        const newColumns = [...paramColumns.filter(e => e.hide === undefined || !e.hide)];
        //Selection column
        const selectionExists = newColumns.some(e => e.id === rowSelection_1.ROW_SELECTION_HEADER_ID);
        if (selection && !selectionExists) {
            logger.debug('Creating the selection header.');
            newColumns.push((0, createSelectionColumn_1.createSelectionColumn)(selection));
            columnsRef.current = newColumns;
        }
        //Detect duplicated
        const duplicateColumns = newColumns.filter((column, i) => {
            return newColumns.findIndex(d => d.id === column.id) !== i;
        });
        if (duplicateColumns.length) {
            throw new Error(`Duplicate columns were found with ids: "${duplicateColumns
                .map(d => d.id)
                .join(', ')}" in the columns array above`);
        }
        //Validate % width to prevent overflow
        let totalWidth = newColumns.reduce((acc, e) => {
            if (e.width && typeof e.width === 'string' && e.width.includes('%')) {
                return acc + parseFloat(e.width.replace('%', '').trim());
            }
            return acc;
        }, 0);
        // Cap if total width is bigger than 100
        if (totalWidth > 100) {
            logger.warn(`Column widths cannot pass 100%, please review your configuration. Received ${totalWidth}% out of 100%, capping to 100%`);
            totalWidth = 100;
        }
        //Check and maybe validate if needed
        const transformedHeaders = newColumns.map(e => (Object.assign(Object.assign({}, e), { colSpan: 0, isNested: false })));
        if (paramNestedHeaders) {
            logger.debug('Grid detected nested headers.');
            //Check if any row passes the limit of spanning
            paramNestedHeaders.forEach((row, i) => {
                const spanSize = row.reduce((acc, e) => { var _a; return acc + ((_a = e.colSpan) !== null && _a !== void 0 ? _a : 0); }, 0);
                if (spanSize > columns.length) {
                    throw new Error(`Span size is bigger than the main headers size, please review the configuration at row: ${i}`);
                }
            });
            const nestedTransform = paramNestedHeaders.map(rows => rows.map((e, i) => ({
                id: `nested-${i}`,
                accessor: '',
                title: e.title,
                colSpan: e.colSpan,
                className: e.className,
                isNested: true,
            })));
            //Here we combine with headers and insert the dummy cells
            return setGridHeaders((0, gridWrapper_1.insertDummyCells)([...nestedTransform, transformedHeaders]));
        }
        const updatedHeaders = (0, gridWrapper_1.insertDummyCells)([transformedHeaders]);
        //Only one level is necessary
        setGridHeaders(updatedHeaders);
        //apiRef.current.dispatchEvent(DATA_CHANGED, { updatedHeaders })
    }, [columns.length, logger, selection]);
    const updateColumns = (0, react_1.useCallback)((updatedColumns) => {
        logger.debug('Updating columns.');
        columnsRef.current = nestedColumnsEnabled
            ? (0, flatExpandedColumns_1.flatExpandedColumns)(originalColumnsRef.current, apiRef)
            : updatedColumns;
        apiRef.current.dispatchEvent(api_1.COLUMNS_CHANGED, { columns: columnsRef.current });
        createGridHeaders({ columns: columnsRef.current, nestedHeaders: nestedHeadersRef.current });
    }, [apiRef, createGridHeaders, logger, nestedColumnsEnabled]);
    //Refresh the data if any dependency change
    (0, react_1.useEffect)(() => {
        //Ensure the grid api is initialized first
        if (!initialised) {
            return;
        }
        originalColumnsRef.current = columns;
        updateColumns(columns);
    }, [columns, updateColumns, initialised]);
    (0, react_1.useEffect)(() => {
        /**
         * @todo the comented code has a conflict when we change the data in table (onCellChange)
         * whenever we change a cell, it collapses the columns
         */
        // columnsRef.current = columns
        // nestedHeadersRef.current = nestedHeaders
        // createGridHeaders({ columns, nestedHeaders })
        //TODO review if this might give us trouble in case of only nested headers changing
        //apiRef.current.dispatchEvent(COLUMNS_CHANGED, { columns })
    }, [apiRef, columns, createGridHeaders, nestedHeaders]);
    const getColumnAt = (0, react_1.useCallback)((index) => columnsRef.current[index], []);
    const getColumnById = (0, react_1.useCallback)((id) => columnsRef.current.find(e => e.id === id), []);
    const getColumnCount = (0, react_1.useCallback)(() => columnsRef.current.length, []);
    const getColumns = (0, react_1.useCallback)(() => columnsRef.current, []);
    const getColumnIndex = (0, react_1.useCallback)((id) => columnsRef.current.findIndex(e => e.id === id), []);
    const getOriginalColumns = (0, react_1.useCallback)(() => originalColumnsRef.current, []);
    const onCollapsesChange = (0, react_1.useCallback)(() => {
        logger.debug('Collapses have changed');
        updateColumns(columnsRef.current);
    }, [logger, updateColumns]);
    const columnApi = {
        getColumnAt,
        getColumnCount,
        getColumnById,
        updateColumns,
        getColumns,
        getColumnIndex,
        getOriginalColumns,
    };
    (0, api_1.useApiExtends)(apiRef, columnApi, 'Data Api');
    (0, api_1.useApiEventHandler)(apiRef, api_1.COLLAPSES_COLUMNS_CHANGED, onCollapsesChange);
    return {
        gridHeaders,
        columns: columnsRef.current,
    };
}
exports.useHeaders = useHeaders;
