"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useData = void 0;
const react_1 = require("react");
const api_1 = require("../api");
const createData_1 = require("./createData");
const logger_1 = require("../logger");
const flatExpandedRows_1 = require("./flatExpandedRows");
/**
 * `useData` handles all the rows and cells operations of this plugin
 */
function useData({ rows, selection, initialised, apiRef, nestedRowsEnabled }) {
    const logger = (0, logger_1.useLogger)('useData');
    const rowsRef = (0, react_1.useRef)(rows);
    const originalRowsRef = (0, react_1.useRef)(rows);
    const cellsRef = (0, react_1.useRef)([]);
    const [cells, setCells] = (0, react_1.useState)([]);
    const selectionRef = (0, react_1.useRef)(selection);
    (0, react_1.useEffect)(() => {
        selectionRef.current = selection;
    }, [selection]);
    const onRowsChangeHandle = (0, react_1.useCallback)((params) => {
        const updatedData = (0, createData_1.createData)(Object.assign(Object.assign({}, params), { apiRef,
            selection }));
        if (!updatedData) {
            return logger.error('No data has been returned from createData, please review the dependencies');
        }
        cellsRef.current = updatedData;
        setCells(updatedData);
        apiRef.current.dispatchEvent(api_1.DATA_CHANGED, { updatedData });
    }, [apiRef, selection, logger]);
    const updateRows = (0, react_1.useCallback)((updatedRows) => {
        logger.debug('Updating rows.');
        //Update current rows but if nestedRows are enabled then we need to read from the original
        rowsRef.current = nestedRowsEnabled
            ? (0, flatExpandedRows_1.flatExpandedRows)(originalRowsRef.current, apiRef)
            : updatedRows;
        apiRef.current.dispatchEvent(api_1.ROWS_CHANGED, { rows: rowsRef.current });
        onRowsChangeHandle({ rows: rowsRef.current });
    }, [apiRef, logger, onRowsChangeHandle, nestedRowsEnabled]);
    //Refresh the data if any dependency change
    (0, react_1.useEffect)(() => {
        //Ensure the grid api is initialized first
        if (!initialised) {
            return;
        }
        originalRowsRef.current = rows;
        updateRows(rows);
    }, [rows, updateRows, initialised]);
    const onRowSelectionChange = (0, react_1.useCallback)(() => {
        logger.debug('Row selection changed.');
        onRowsChangeHandle({ rows: rowsRef.current });
    }, [logger, onRowsChangeHandle]);
    const onCollapsesChange = (0, react_1.useCallback)(() => {
        logger.debug('Collapses have changed');
        updateRows(rowsRef.current);
    }, [logger, updateRows]);
    const getRowAt = (0, react_1.useCallback)((index) => rowsRef.current[index], []);
    const getRows = (0, react_1.useCallback)(() => rowsRef.current, []);
    const getRowsCount = (0, react_1.useCallback)(() => rowsRef.current.length, []);
    const getRowById = (0, react_1.useCallback)((id) => {
        var _a;
        if (!selectionRef.current || !((_a = selectionRef.current) === null || _a === void 0 ? void 0 : _a.key)) {
            logger.warn('No selection key to getRowById');
            return undefined;
        }
        return rowsRef.current.find(e => String(e[selectionRef.current.key]) === id);
    }, [logger]);
    const getRowsWithFilter = (0, react_1.useCallback)((predicate, thisArg) => {
        return rowsRef.current.filter(predicate, thisArg);
    }, []);
    const getRowIndex = (0, react_1.useCallback)((id) => {
        return rowsRef.current.findIndex(e => { var _a; return String(e[(_a = selection === null || selection === void 0 ? void 0 : selection.key) !== null && _a !== void 0 ? _a : '']) === id; });
    }, [selection === null || selection === void 0 ? void 0 : selection.key]);
    const getOriginalRows = (0, react_1.useCallback)(() => originalRowsRef.current, []);
    const getCells = (0, react_1.useCallback)(() => cellsRef.current, []);
    const rowApi = {
        getCells,
        getRows,
        getOriginalRows,
        getRowsCount,
        getRowAt,
        getRowById,
        getRowsWithFilter,
        getRowIndex,
        updateRows,
    };
    (0, api_1.useApiExtends)(apiRef, rowApi, 'Data API');
    (0, api_1.useApiEventHandler)(apiRef, api_1.ROW_SELECTION_CHANGE, onRowSelectionChange);
    (0, api_1.useApiEventHandler)(apiRef, api_1.COLLAPSES_CHANGED, onCollapsesChange);
    (0, api_1.useApiEventHandler)(apiRef, api_1.COLLAPSES_COLUMNS_CHANGED, onCollapsesChange);
    return { cells, rows: rowsRef.current };
}
exports.useData = useData;
