"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useEditorManager = void 0;
const tslib_1 = require("tslib");
const react_1 = require("react");
const rowSelection_1 = require("../rowSelection");
const columnGrid_1 = require("../columnGrid");
const helpers_1 = require("../helpers");
const api_1 = require("../api");
const clsx_1 = tslib_1.__importDefault(require("clsx"));
const logger_1 = require("../logger");
const value_equal_1 = tslib_1.__importDefault(require("value-equal"));
const utils_1 = require("./utils");
/**
 * Provides a way to manage the editors and also returns the active editor node
 * This hook controls the editing states, interacts with useNavigation hook and also manages the commit/cancel cycle of
 * an editor
 */
function useEditorManager({ onCellChange, apiRef }) {
    const logger = (0, logger_1.useLogger)('useEditorManager');
    const editorRef = (0, react_1.useRef)();
    const state = (0, react_1.useRef)(null);
    const [editorNode, setEditorNode] = (0, react_1.useState)(null);
    (0, react_1.useEffect)(() => {
        if (editorNode) {
            apiRef.current.dispatchEvent('CELL_BEGIN_EDITING', apiRef.current.getSelectedCoords());
        }
    }, [editorNode, apiRef]);
    /**
     * Closes the existing editor without saving anything
     */
    const stopEditing = (0, react_1.useCallback)((params) => {
        var _a, _b, _c, _d, _e, _f, _g;
        const editorState = state.current;
        if (!editorState) {
            return;
        }
        if ((params === undefined || params.save) && editorState) {
            const newValue = (_b = (_a = editorRef.current) === null || _a === void 0 ? void 0 : _a.getValue()) !== null && _b !== void 0 ? _b : undefined;
            if (newValue === undefined) {
                state.current = null;
                editorRef.current = null;
                apiRef.current.dispatchEvent('CELL_STOP_EDITING', {
                    colIndex: editorState.colIndex,
                    rowIndex: editorState.rowIndex,
                });
                return setEditorNode(null);
            }
            const isValid = (_d = (_c = editorState.validatorHook) === null || _c === void 0 ? void 0 : _c.call(editorState, newValue)) !== null && _d !== void 0 ? _d : true;
            if (!isValid) {
                editorRef.current = null;
                state.current = null;
                apiRef.current.dispatchEvent('CELL_STOP_EDITING', {
                    colIndex: editorState.colIndex,
                    rowIndex: editorState.rowIndex,
                });
                return setEditorNode(null);
            }
            //Compare the values before dispatch
            if (!(0, value_equal_1.default)(newValue, editorState.initialValue)) {
                //If we have the hook, invoke it before we proceed
                if (((_e = state.current) === null || _e === void 0 ? void 0 : _e.shouldSaveHook) &&
                    !((_g = (_f = state.current).shouldSaveHook) === null || _g === void 0 ? void 0 : _g.call(_f, editorState.initialValue, newValue))) {
                    editorRef.current = null;
                    state.current = null;
                    apiRef.current.dispatchEvent('CELL_STOP_EDITING', {
                        colIndex: editorState.colIndex,
                        rowIndex: editorState.rowIndex,
                    });
                    return setEditorNode(null);
                }
                const row = apiRef.current.getRowAt(editorState.rowIndex);
                const column = apiRef.current.getColumnAt(editorState.colIndex);
                if (!row) {
                    logger.warn(`Row not found at ${editorState.rowIndex} when attempting to invoke onCellChange`);
                }
                else if (!column) {
                    logger.warn(`Column not found at ${editorState.colIndex} when attempting to invoke onCellChange`);
                }
                else {
                    onCellChange === null || onCellChange === void 0 ? void 0 : onCellChange({
                        coords: {
                            rowIndex: editorState.rowIndex,
                            colIndex: editorState.colIndex,
                        },
                        previousValue: editorState.initialValue,
                        newValue,
                        row,
                        column,
                    }, 'editor');
                }
            }
        }
        editorRef.current = null;
        state.current = null;
        apiRef.current.dispatchEvent('CELL_STOP_EDITING', {
            colIndex: editorState.colIndex,
            rowIndex: editorState.rowIndex,
        });
        setEditorNode(null);
    }, [apiRef, logger, onCellChange]);
    //Detect if the active editing row/column has been deleted
    (0, react_1.useEffect)(() => {
        if (editorNode && state.current) {
            const target = apiRef.current.getRowAt(state.current.rowIndex);
            const column = apiRef.current.getColumnAt(state.current.colIndex);
            if (!target || !column) {
                stopEditing({ save: false });
            }
        }
    }, [apiRef, editorNode, stopEditing]);
    const validateEditorRef = (0, react_1.useCallback)((editorRef) => {
        if (!editorRef) {
            logger.warn(`
				useImperativeHandle is missing on the editor component OR has some misconfiguration. Editor reference is not defined therefore
				its not possible to start editing at the current cell. Please review your setup
			`);
            return false;
        }
        if (!editorRef.getValue || !(0, helpers_1.isFunctionType)(editorRef.getValue)) {
            logger.warn('Editor reference "getValue()" method is invalid, not a function or undefined, please review your setup');
            return false;
        }
        return true;
    }, [logger]);
    //Invoked when the editor mounts on DOM
    const onRefMount = (0, react_1.useCallback)((ref) => {
        if (!ref) {
            return;
        }
        validateEditorRef(ref);
        editorRef.current = ref;
    }, [validateEditorRef]);
    /**
     * Starts editing in a given cell considering multiple configurations
     * @param   coords  NavigationCoords
     * @param   targetElement Element
     * @param   refresh Flag indicating whether we bypass or not the check for current editing. Use this carefully and only when you
     * want to reload the editor
     */
    const beginEditing = (0, react_1.useCallback)(({ coords, targetElement, defaultKey }) => {
        var _a, _b, _c, _d, _e, _f;
        logger.debug(`Begin editing invoked for coords: ${(0, helpers_1.coordinatesToString)(coords)}`);
        //Validate if is editing but in the same coords
        if (((_a = state.current) === null || _a === void 0 ? void 0 : _a.rowIndex) === coords.rowIndex &&
            ((_b = state.current) === null || _b === void 0 ? void 0 : _b.colIndex) === coords.colIndex) {
            return logger.debug(`Already editing at: ${(0, helpers_1.coordinatesToString)(coords)}`);
        }
        const column = apiRef.current.getColumnAt(coords.colIndex);
        if (!column) {
            return logger.warn(`Column not found at ${coords.colIndex}, therefore we can't start editing.`);
        }
        if (column.id === rowSelection_1.ROW_SELECTION_HEADER_ID) {
            return logger.debug(`Cannot edit row selection id: ${rowSelection_1.ROW_SELECTION_HEADER_ID}`);
        }
        // eslint-disable-next-line no-nested-ternary
        const isReadOnly = column.readOnly
            ? (0, helpers_1.isFunctionType)(column.readOnly)
                ? column.readOnly(coords)
                : column.readOnly
            : false;
        if (isReadOnly) {
            return logger.debug(`Cell is readOnly therefore cannot edit at: ${(0, helpers_1.coordinatesToString)(coords)}`);
        }
        const row = apiRef.current.getRowAt(coords.rowIndex);
        if (!row) {
            return logger.warn(`Row not found at ${coords.rowIndex}, therefore we can't start editing at column: ${column.id}`);
        }
        // If any valid key is pressed we override the existing text by default for the editor
        // eslint-disable-next-line no-nested-ternary
        const value = row[column.accessor] ? defaultKey || row[column.accessor] : defaultKey || '';
        const initialValue = (_c = row[column.accessor]) !== null && _c !== void 0 ? _c : '';
        let componentProps;
        const columnEditorProps = column.editorProps;
        if ((0, helpers_1.isFunctionType)(columnEditorProps === null || columnEditorProps === void 0 ? void 0 : columnEditorProps.componentProps)) {
            componentProps = columnEditorProps === null || columnEditorProps === void 0 ? void 0 : columnEditorProps.componentProps(row, column);
        }
        if (columnEditorProps === null || columnEditorProps === void 0 ? void 0 : columnEditorProps.componentProps) {
            componentProps = columnEditorProps === null || columnEditorProps === void 0 ? void 0 : columnEditorProps.componentProps;
        }
        const editorProps = {
            anchorRef: targetElement,
            value,
            maxLength: (_d = column.maxLength) !== null && _d !== void 0 ? _d : 500,
            additionalProps: Object.assign(Object.assign({}, column.editorProps), { className: (0, clsx_1.default)((_e = apiRef.current.getTheme()) === null || _e === void 0 ? void 0 : _e.editorClass, (_f = column.editorProps) === null || _f === void 0 ? void 0 : _f.className), componentProps }),
            stopEditing,
            validatorHook: column.validatorHook,
            apiRef,
        };
        const editor = (0, utils_1.getEditorComponent)(row, column, editorProps, onRefMount);
        if (!editor) {
            return logger.warn('Editor not found on active coordinates, returned undefined or null');
        }
        state.current = {
            node: editor,
            rowIndex: coords.rowIndex,
            colIndex: coords.colIndex,
            initialValue,
            targetElement,
            validatorHook: column.validatorHook,
            isPopup: column.editor !== undefined || column.type === columnGrid_1.ColumnCellType.Calendar,
            shouldSaveHook: column.shouldSaveHook,
        };
        setEditorNode(editor);
    }, [logger, apiRef, stopEditing, onRefMount]);
    const getEditorState = (0, react_1.useCallback)(() => {
        return state.current;
    }, []);
    const editorManagerApi = {
        beginEditing,
        stopEditing,
        getEditorState,
    };
    (0, api_1.useApiExtends)(apiRef, editorManagerApi, 'EditorManagerApi');
    return editorNode;
}
exports.useEditorManager = useEditorManager;
