"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const react_virtualized_1 = require("react-virtualized");
const CellMeasureWrapper_1 = tslib_1.__importDefault(require("../cellMeasurer/CellMeasureWrapper"));
const keyboard_1 = require("../keyboard");
const clsx_1 = tslib_1.__importDefault(require("clsx"));
const styles_1 = require("@mui/styles");
const types_1 = require("../types");
const logger_1 = require("../logger");
const KeyboardArrowRight_1 = tslib_1.__importDefault(require("@mui/icons-material/KeyboardArrowRight"));
const KeyboardArrowDown_1 = tslib_1.__importDefault(require("@mui/icons-material/KeyboardArrowDown"));
const api_1 = require("../api");
const useStyles = (0, styles_1.makeStyles)((theme) => ({
    bodyContainer: {
        outline: 'none',
    },
    cellDefaultStyle: {
        display: 'flex',
        boxSizing: 'border-box',
        '&:focus': {
            outline: 0,
            border: 0,
        },
    },
    disabledCell: {
        cursor: 'default',
        pointerEvents: 'none', //no events for this cell
    },
    suppressHorizontalOverflow: {
        overflowX: 'hidden',
    },
}));
const GridWrapper = react_1.default.memo(({ rows, data, getColumnWidth, isMerged, apiRef, columns, nestedRowsProps, coords, theme, fixedRowHeight, fixedRowWidth, scrollLeft, selection, scrollToAlignment, stretchMode, onScroll, mergeCells, overscanRowCount, overscanColumnCount, minRowHeight, minColumnWidth, columnCount, height, width, highlightBorderColor, rowHeight, noContentOverlay, onRowCollapse, displayCollapseIcon = true, coreId, }) => {
    const logger = (0, logger_1.useLogger)('GridWrapper');
    const cache = (0, react_1.useMemo)(() => {
        const isFixedCellHeight = fixedRowHeight && rowHeight;
        const options = {
            defaultWidth: minColumnWidth,
            defaultHeight: isFixedCellHeight ? rowHeight : minRowHeight,
            fixedWidth: fixedRowWidth !== null && fixedRowWidth !== void 0 ? fixedRowWidth : true,
            fixedHeight: fixedRowHeight,
            minHeight: isFixedCellHeight ? undefined : minRowHeight,
            minWidth: minColumnWidth,
        };
        /**
         * Used to skip calculations in a faster way when we have fixed height
         * @see https://github.com/bvaughn/react-virtualized/blob/master/docs/CellMeasurer.md
         */
        if (isFixedCellHeight) {
            options.keyMapper = () => 1;
        }
        return new react_virtualized_1.CellMeasurerCache(options);
    }, [fixedRowHeight, fixedRowWidth, minColumnWidth, minRowHeight, rowHeight]);
    const cacheRef = (0, react_1.useRef)(cache);
    const classes = useStyles();
    const gridRef = (0, react_1.useRef)(null);
    const loggerRef = (0, react_1.useRef)(logger);
    const coordsRef = (0, react_1.useRef)(coords);
    (0, react_1.useEffect)(() => {
        coordsRef.current = coords;
    }, [coords]);
    (0, react_1.useEffect)(() => {
        loggerRef.current = logger;
    }, [logger]);
    const recomputeSizes = (0, react_1.useCallback)(() => {
        var _a;
        loggerRef.current.debug('Recomputing sizes.');
        cacheRef.current.clearAll();
        (_a = gridRef.current) === null || _a === void 0 ? void 0 : _a.recomputeGridSize();
    }, []);
    (0, api_1.useApiEventHandler)(apiRef, 'GRID_RESIZE', recomputeSizes);
    (0, api_1.useApiEventHandler)(apiRef, 'DATA_CHANGED', recomputeSizes);
    (0, api_1.useApiEventHandler)(apiRef, 'COLUMNS_CHANGED', recomputeSizes);
    const isMergeCellsEnabled = Array.isArray(mergeCells) && mergeCells.length > 0;
    const activeRowPathCoordinates = (0, react_1.useMemo)(() => {
        //Skip the heavy calculation process if its disabled
        if (!isMergeCellsEnabled) {
            return [];
        }
        const coordinates = [];
        const { rowIndex } = coords;
        //Create the cell coordinates position with all columns on the active rowIndex
        const temporaryRowCoordinates = columns.map((_, colIndex) => {
            return { rowIndex, colIndex };
        });
        //Check each temporary row coordinate
        temporaryRowCoordinates.forEach(e => {
            if (isMerged && isMerged(e)) {
                const parent = apiRef.current.getMergeParentCoords(e);
                if (!parent) {
                    logger.warn(`Parent not found for coordinates`);
                    logger.info({ parent, currentCoordinate: e });
                }
                else {
                    coordinates.push(parent);
                }
            }
            else {
                coordinates.push(e);
            }
        });
        return coordinates;
    }, [apiRef, columns, coords, isMergeCellsEnabled, isMerged, logger]);
    /**
     * Checks if the given coordinates can use the currentClassName
     * @param rowIndex
     * @param colIndex
     */
    const isCellRowActive = (0, react_1.useCallback)(({ rowIndex, colIndex }) => {
        //Check directly on the coords
        if (!isMergeCellsEnabled) {
            return coords.rowIndex === rowIndex && coords.colIndex === colIndex;
        }
        return activeRowPathCoordinates.some(e => e.rowIndex === rowIndex && e.colIndex === colIndex);
    }, [isMergeCellsEnabled, coords, activeRowPathCoordinates]);
    const renderCell = (0, react_1.useCallback)(({ style, cell, ref, rowIndex, columnIndex }) => {
        var _a, _b;
        const isSelected = rowIndex === coords.rowIndex && columnIndex === coords.colIndex;
        const navigationDisabled = (_a = columns[0][columnIndex]) === null || _a === void 0 ? void 0 : _a.disableNavigation;
        const column = columns[columnIndex];
        const row = rows[rowIndex];
        //Dummy zIndex is 0 and a spanned cell has 5 but a normal cell has 1
        const defaultZIndex = cell.dummy ? 0 : 1;
        const zIndex = (cell.rowSpan || cell.colSpan) && !cell.dummy ? 5 : defaultZIndex;
        const isLineActive = isCellRowActive({ rowIndex, colIndex: columnIndex });
        const cellStyle = Object.assign({}, style);
        if (isSelected) {
            //Ensure there are no other borders
            cellStyle.borderLeft = '0px';
            cellStyle.borderRight = '0px';
            cellStyle.borderTop = '0px';
            cellStyle.borderBottom = '0px';
            cellStyle.border = highlightBorderColor
                ? `1px solid ${highlightBorderColor}`
                : '1px solid blue';
        }
        else if (!theme || (!theme.cellClass && !cell.dummy)) {
            //Bind default border and clear other borders
            cellStyle.borderLeft = '0px';
            cellStyle.borderRight = '0px';
            cellStyle.borderTop = '0px';
            cellStyle.borderBottom = '0px';
            cellStyle.border = '1px solid rgb(204, 204, 204)';
        }
        //Add default dummy color when a cell class is not provided by default
        if (!cell.dummy && !(theme === null || theme === void 0 ? void 0 : theme.cellClass)) {
            cellStyle.backgroundColor = 'white';
        }
        /**
         * @todo We need to check if the row is a dummy but its parent dummy is not anymore visible (we need to pass the content to the last visible child)
         * e.:g
         * dummy 1 has a rowspan of total 3 but none of its parent are visible, so dummy 3 assume the children value and highlight
         * of the parent because there is none visible
         * */
        let cellClassName = (0, clsx_1.default)(classes.cellDefaultStyle, theme === null || theme === void 0 ? void 0 : theme.cellClass, typeof column.cellClassName === 'function'
            ? column.cellClassName({ row, column })
            : column.cellClassName);
        if (isLineActive && !cell.dummy && (theme === null || theme === void 0 ? void 0 : theme.currentRowClass)) {
            cellClassName = (0, clsx_1.default)(cellClassName, theme === null || theme === void 0 ? void 0 : theme.currentRowClass);
        }
        if (navigationDisabled && !cell.dummy && (theme === null || theme === void 0 ? void 0 : theme.disabledCellClass)) {
            cellClassName = (0, clsx_1.default)(cellClassName, classes.disabledCell, theme === null || theme === void 0 ? void 0 : theme.disabledCellClass);
        }
        if (selection && selection.cellClassName) {
            const isRowSelected = apiRef.current.isRowSelected(row === null || row === void 0 ? void 0 : row[selection.key]);
            if (isRowSelected) {
                cellClassName = (0, clsx_1.default)(cellClassName, selection.cellClassName);
            }
        }
        const wrapper = child => {
            //Add navigationProps in case its a normal cell navigable
            const navigationProps = cell.dummy
                ? {}
                : (0, keyboard_1.createCellQueryProperties)({
                    colIndex: columnIndex,
                    rowIndex,
                    role: 'cell',
                    accessor: column.accessor,
                });
            const style = Object.assign(Object.assign({}, cellStyle), { justifyContent: (cell === null || cell === void 0 ? void 0 : cell.dummy) ? 'top' : 'center', zIndex });
            return (react_1.default.createElement("div", Object.assign({}, navigationProps, { className: cellClassName, style: style, ref: ref }), child));
        };
        //If nestedRows are enabled we need to render our utility
        if (nestedRowsProps.nestedRows) {
            const id = String(row[apiRef.current.selectionKey]);
            const depth = apiRef.current.getRowDepth(id);
            const nestedMargin = ((_b = nestedRowsProps.nestedRowMargin) !== null && _b !== void 0 ? _b : 10) * depth;
            //Parent collapse renders an additional layer with collapse controls
            if (row.__children !== undefined && columnIndex === 0) {
                const iconStyle = {
                    cursor: 'pointer',
                    right: 0,
                    position: 'absolute',
                };
                const isRowExpanded = apiRef.current.isRowExpanded(id);
                const handleCollapseClick = () => {
                    if (!id) {
                        return logger.error(`Row index ${rowIndex} does not have a key in order to toggle collapse!`);
                    }
                    //Select the target row if not selected
                    if (coords.rowIndex !== rowIndex) {
                        apiRef.current.selectCell(Object.assign(Object.assign({}, coords), { rowIndex }));
                    }
                    apiRef.current.toggleRowExpand(id);
                    if (onRowCollapse) {
                        onRowCollapse(id);
                    }
                };
                const renderExpandOrCollapseIcon = () => {
                    if (nestedRowsProps.iconRenderer) {
                        return nestedRowsProps.iconRenderer(handleCollapseClick, isRowExpanded);
                    }
                    return isRowExpanded ? (react_1.default.createElement(KeyboardArrowDown_1.default, { onClick: handleCollapseClick, fontSize: 'small', style: iconStyle })) : (react_1.default.createElement(KeyboardArrowRight_1.default, { onClick: handleCollapseClick, fontSize: 'small', style: iconStyle }));
                };
                const component = (react_1.default.createElement("div", { style: depth > 1 ? { marginLeft: nestedMargin } : {} },
                    cell.value,
                    displayCollapseIcon ? renderExpandOrCollapseIcon() : null));
                return wrapper(component);
            }
            //Nested rows need to be wrapped on the first cell with a marginLeft
            if (depth > 1 && columnIndex === 0) {
                const component = react_1.default.createElement("div", { style: { marginLeft: nestedMargin } }, cell.value);
                return wrapper(component);
            }
        }
        return wrapper(cell.value);
    }, [
        coords,
        columns,
        rows,
        isCellRowActive,
        theme,
        classes.cellDefaultStyle,
        classes.disabledCell,
        selection,
        nestedRowsProps,
        highlightBorderColor,
        apiRef,
        displayCollapseIcon,
        onRowCollapse,
        logger,
    ]);
    const cellRenderer = (0, react_1.useCallback)((_a) => {
        var _b;
        var { rowIndex, columnIndex, key, parent, style } = _a, otherArgs = tslib_1.__rest(_a, ["rowIndex", "columnIndex", "key", "parent", "style"]);
        const cell = (_b = data[rowIndex]) === null || _b === void 0 ? void 0 : _b[columnIndex];
        const rendererProps = Object.assign(Object.assign({}, otherArgs), { style,
            rowIndex,
            columnIndex,
            cell,
            getColumnWidth });
        return cell ? (react_1.default.createElement(CellMeasureWrapper_1.default, { cache: cache, columnIndex: columnIndex, key: key, parent: parent, rowIndex: rowIndex, rowSpan: cell.rowSpan, colSpan: cell.colSpan, cellRenderer: renderCell, style: Object.assign(Object.assign(Object.assign({}, style), cell.style), { width: getColumnWidth({
                    index: columnIndex,
                }), userSelect: 'none' }), rendererProps: rendererProps })) : null;
    }, [data, getColumnWidth, cache, renderCell]);
    const onRefMount = (0, react_1.useCallback)(instance => {
        var _a;
        gridRef.current = instance;
        (_a = document.getElementById('core-grid')) === null || _a === void 0 ? void 0 : _a.focus();
    }, []);
    const onSectionRendered = (0, react_1.useCallback)((params) => {
        const editorState = apiRef.current.getEditorState();
        /** @todo Store in a ref the visible rows/columns **/
        // Check if the editing coords are within the visible range
        if (editorState) {
            if (editorState.rowIndex < params.rowStartIndex ||
                editorState.rowIndex > params.rowStopIndex) {
                apiRef.current.stopEditing({ save: false });
            }
            else if (editorState.colIndex < params.columnStartIndex ||
                editorState.colIndex > params.columnStopIndex) {
                apiRef.current.stopEditing({ save: false });
            }
        }
    }, [apiRef]);
    const getGridRef = (0, react_1.useCallback)(() => {
        return gridRef.current;
    }, []);
    const gridWrapperApi = { getGridRef };
    (0, api_1.useApiExtends)(apiRef, gridWrapperApi, 'GridWrapperApi');
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(react_virtualized_1.Grid, { id: coreId, className: stretchMode !== types_1.StretchMode.None
                ? (0, clsx_1.default)(classes.bodyContainer, classes.suppressHorizontalOverflow)
                : classes.bodyContainer, ref: onRefMount, cellRenderer: cellRenderer, deferredMeasurementCache: cache, rowHeight: cache.rowHeight, rowCount: rows.length, columnCount: columnCount, columnWidth: getColumnWidth, overscanRowCount: overscanRowCount !== null && overscanRowCount !== void 0 ? overscanRowCount : 10, overscanColumnCount: overscanColumnCount !== null && overscanColumnCount !== void 0 ? overscanColumnCount : 5, onSectionRendered: onSectionRendered, scrollToAlignment: scrollToAlignment, onScroll: onScroll, scrollLeft: scrollLeft, height: height, width: width, noContentRenderer: noContentOverlay, scrollToColumn: coords.colIndex, scrollToRow: coords.rowIndex, tabIndex: -1 })));
});
exports.default = GridWrapper;
