"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useKeyboard = void 0;
const tslib_1 = require("tslib");
const react_1 = require("react");
const utils_1 = require("./utils");
const helpers_1 = require("../helpers");
const editorManager_1 = require("../editorManager");
const clipboardy = tslib_1.__importStar(require("clipboardy"));
const columnGrid_1 = require("../columnGrid");
const dayjs_1 = tslib_1.__importDefault(require("dayjs"));
const rowSelection_1 = require("../rowSelection");
const lodash_1 = require("lodash");
const api_1 = require("../api");
const logger_1 = require("../logger");
const resolveDynamicCallback_1 = require("../helpers/resolveDynamicCallback");
function useKeyboard({ defaultCoords, suppressControls, onCellChange, onCreateRow, apiRef, initialised, }) {
    const logger = (0, logger_1.useLogger)(useKeyboard.name);
    const coordsRef = (0, react_1.useRef)(defaultCoords);
    const [coords, setCoords] = (0, react_1.useState)(defaultCoords);
    const delayEditorDebounce = (0, react_1.useRef)(null);
    const [wasClicked, setWasCliked] = (0, react_1.useState)(false);
    const onCellBeginEditing = (0, react_1.useCallback)(() => {
        if (delayEditorDebounce.current) {
            delayEditorDebounce.current.cancel();
        }
    }, []);
    const onRowsChanged = (0, react_1.useCallback)(({ rows }) => {
        const target = rows[coordsRef.current.rowIndex];
        if (!target) {
            coordsRef.current = defaultCoords;
            apiRef.current.dispatchEvent('CELL_NAVIGATION_CHANGED', { coords: coordsRef.current });
            setCoords(defaultCoords);
        }
    }, [apiRef, defaultCoords]);
    //Cancels the debounce if the editor is prematurely open
    (0, api_1.useApiEventHandler)(apiRef, 'CELL_BEGIN_EDITING', onCellBeginEditing);
    //Subscribe for data changes to ensure the selected coordinates exist
    (0, api_1.useApiEventHandler)(apiRef, 'ROWS_CHANGED', onRowsChanged);
    //Cleanup the debounce on unmount
    (0, react_1.useEffect)(() => () => {
        var _a;
        (_a = delayEditorDebounce.current) === null || _a === void 0 ? void 0 : _a.cancel();
    }, []);
    /**
     * Recursively looks for the next navigable cell
     * @param currentIndex
     * @param direction
     */
    const findNextNavigableColumnIndex = (0, react_1.useCallback)((currentIndex, direction) => {
        const nextIndex = direction === 'right' ? currentIndex + 1 : currentIndex - 1;
        const nextCol = apiRef.current.getColumnAt(nextIndex);
        //Fallback the current in case it was not found
        if (!nextCol) {
            return currentIndex;
        }
        if (nextCol.disableNavigation !== undefined) {
            const { rowIndex } = apiRef.current.getSelectedCoords();
            const disabled = (0, helpers_1.isFunctionType)(nextCol.disableNavigation)
                ? nextCol.disableNavigation({ colIndex: nextIndex, rowIndex })
                : nextCol.disableNavigation;
            if (disabled) {
                return findNextNavigableColumnIndex(nextIndex, direction);
            }
        }
        return nextIndex;
    }, [apiRef]);
    const selectCell = (0, react_1.useCallback)(({ colIndex, rowIndex }, force, targetElement) => {
        logger.debug(`Select cell for coordinates [${rowIndex},${colIndex}]`);
        if (colIndex === undefined || rowIndex === undefined) {
            return logger.error(`Undefined coordinates detected at selectCell [${rowIndex},${colIndex}]. If you wish to remove highlight, you can pass -1, -1 coordinates`);
        }
        //Coordinates when the grid is clicked away
        if ((colIndex === -1 && rowIndex === -1) || force) {
            logger.debug('Force or negative -1 coordinates selected');
            coordsRef.current = { colIndex, rowIndex };
            apiRef.current.dispatchEvent('CELL_NAVIGATION_CHANGED', { coords: coordsRef.current });
            return setCoords({ colIndex, rowIndex });
        }
        //Equal selection comparison
        if (coords.colIndex === colIndex && coords.rowIndex === rowIndex) {
            return logger.debug('Coordinates given are equal to the current coordinates');
        }
        const column = apiRef.current.getColumnAt(colIndex);
        if (!column) {
            return logger.warn(`Column not found at index ${colIndex}, review your configuration. Total loaded columns ${apiRef.current.getColumnCount()}`);
        }
        const isDisabled = (0, helpers_1.isFunctionType)(column.disableNavigation)
            ? column.disableNavigation({ rowIndex, colIndex })
            : column.disableNavigation;
        if (isDisabled) {
            return logger.info(`Navigation is disabled for the given column: ${column.id}`);
        }
        const delayEditingOpen = column.delayEditorOpen;
        //Cleanup
        if (delayEditorDebounce.current) {
            logger.info('Delay editor debounce is being cleared');
            delayEditorDebounce.current.cancel();
            delayEditorDebounce.current = null;
        }
        if (delayEditingOpen) {
            delayEditorDebounce.current = (0, lodash_1.debounce)(() => {
                var _a, _b;
                logger.info('Debounce has been invoked');
                delayEditorDebounce.current = null;
                if (colIndex < 0 || rowIndex < 0) {
                    return logger.info("Debounce couldn't start editor at negative coordinates");
                }
                const selector = (0, utils_1.createCellQuerySelector)({ rowIndex, colIndex });
                const target = targetElement !== null && targetElement !== void 0 ? targetElement : (_b = (_a = apiRef.current.rootElementRef) === null || _a === void 0 ? void 0 : _a.current) === null || _b === void 0 ? void 0 : _b.querySelector(selector);
                if (!target) {
                    return logger.warn((0, utils_1.createSelectorElementNotFoundWarning)({ rowIndex, colIndex }));
                }
                apiRef.current.beginEditing({
                    coords: { rowIndex, colIndex },
                    targetElement: target,
                });
            }, delayEditingOpen);
            delayEditorDebounce.current();
        }
        coordsRef.current = { colIndex, rowIndex };
        apiRef.current.dispatchEvent('CELL_NAVIGATION_CHANGED', { coords: coordsRef.current });
        setCoords({ colIndex, rowIndex });
    }, [logger, coords, apiRef]);
    const handleCellPaste = (0, react_1.useCallback)((column, row, currentValue) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        const editingSource = 'paste';
        try {
            logger.debug('[handleCellPaste] Reading value from clipboard');
            let text = yield clipboardy.read();
            //Check the text length if passes the maxLength allowed, if so we cut
            if (column.maxLength && text.length > column.maxLength) {
                text = text.slice(0, column.maxLength);
            }
            if (column.validatorHook) {
                if (column.validatorHook(text)) {
                    return onCellChange === null || onCellChange === void 0 ? void 0 : onCellChange({
                        coords,
                        previousValue: currentValue,
                        newValue: text,
                        column,
                        row,
                    }, editingSource);
                }
                return;
            }
            //Fallback is the column type
            if (column.type === columnGrid_1.ColumnCellType.Numeric) {
                if (!isNaN(Number(text))) {
                    return onCellChange === null || onCellChange === void 0 ? void 0 : onCellChange({
                        coords,
                        previousValue: currentValue,
                        newValue: text,
                        column,
                        row,
                    }, editingSource);
                }
                return;
            }
            if (column.type === columnGrid_1.ColumnCellType.Calendar) {
                if ((0, dayjs_1.default)(text, 'YYYY-MM-DD').format('YYYY-MM-DD') === text) {
                    return onCellChange === null || onCellChange === void 0 ? void 0 : onCellChange({
                        coords,
                        previousValue: currentValue,
                        newValue: text,
                        column,
                        row,
                    }, editingSource);
                }
                return;
            }
            return onCellChange === null || onCellChange === void 0 ? void 0 : onCellChange({ coords, previousValue: currentValue, newValue: text, column, row }, editingSource);
        }
        catch (ex) {
            logger.error(`[handleCellPaste] ${ex}`);
        }
    }), [coords, logger, onCellChange]);
    const handleCellCut = (0, react_1.useCallback)((currentValue, column, row) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        try {
            logger.debug(`[handleCellCut] Cutting ${String(currentValue)} to clipboard`);
            yield clipboardy.write(String(currentValue));
            const newValue = (0, utils_1.getDefaultValueFromValue)(currentValue);
            if (currentValue === newValue) {
                return;
            }
            onCellChange === null || onCellChange === void 0 ? void 0 : onCellChange({ coords, previousValue: currentValue, newValue, column, row }, 'cut');
        }
        catch (ex) {
            logger.error(`[handleCellCut] ${ex}`);
        }
    }), [coords, logger, onCellChange]);
    const handleEditorOpenControls = (0, react_1.useCallback)((event) => {
        if (event.key === 'Escape') {
            event.preventDefault();
            return apiRef.current.stopEditing({ save: false });
        }
        if (event.key === 'Enter') {
            event.preventDefault();
            return apiRef.current.stopEditing();
        }
    }, [apiRef]);
    const handleControlOrMetaPressedControls = (0, react_1.useCallback)((event, column, row, currentValue) => {
        if (event.key === 'x') {
            event.preventDefault();
            if (column.disableCellCut !== undefined &&
                (0, resolveDynamicCallback_1.resolveDynamicCallback)(column.disableCellCut, { row, column })) {
                return;
            }
            return handleCellCut(currentValue, column, row);
        }
        if (event.key === 'c') {
            event.preventDefault();
            try {
                logger.debug(`[handlePaste] Pasting ${String(currentValue)} to clipboard`);
                return clipboardy.write(String(currentValue));
            }
            catch (ex) {
                logger.error(`[handlePaste]: ${ex}`);
                return;
            }
        }
        if (event.key === 'v') {
            event.preventDefault();
            if (column.disableCellPaste !== undefined &&
                (0, resolveDynamicCallback_1.resolveDynamicCallback)(column.disableCellPaste, { row, column })) {
                return;
            }
            return handleCellPaste(column, row, currentValue);
        }
    }, [handleCellCut, handleCellPaste, logger]);
    /** @todo Might need to consider colSpan **/
    const handleArrowNavigationControls = (0, react_1.useCallback)((event) => {
        if (event.key === 'ArrowDown') {
            event.preventDefault();
            let nextRowIndex = coords.rowIndex + 1;
            //If we have span we need to skip that to the next
            const currentCellSpan = apiRef.current.getSpanProperties(coords);
            if (currentCellSpan) {
                nextRowIndex = coords.rowIndex + currentCellSpan.rowSpan;
            }
            //Ensure we are not out of boundaries
            if ((0, utils_1.isIndexOutOfBoundaries)(nextRowIndex, 0, apiRef.current.getRowsCount() - 1)) {
                return;
            }
            return selectCell({
                rowIndex: nextRowIndex,
                colIndex: coords.colIndex,
            });
        }
        if (event.key === 'ArrowUp') {
            event.preventDefault();
            let nextRowIndex = coords.rowIndex - 1;
            //If we have span we need to skip that to the next
            const isNextMerged = apiRef.current.isMerged({
                rowIndex: nextRowIndex,
                colIndex: coords.colIndex,
            });
            if (isNextMerged) {
                const parentCoords = apiRef.current.getMergeParentCoords({
                    rowIndex: nextRowIndex,
                    colIndex: coords.colIndex,
                });
                if (!parentCoords) {
                    return logger.warn(`[Navigation] Merge group path not correct, returned ${parentCoords} positions instead of the expected coordinates. Please review`);
                }
                //Means we have the parent, parent comes in the first position always
                nextRowIndex = parentCoords.rowIndex;
            }
            //Ensure we are not out of boundaries
            if (nextRowIndex < 0) {
                return;
            }
            return selectCell({
                rowIndex: nextRowIndex,
                colIndex: coords.colIndex,
            });
        }
        if (event.key === 'ArrowRight' || (event.key === 'Tab' && !event.shiftKey)) {
            event.preventDefault();
            let nextColIndex = coords.colIndex + 1;
            if ((0, utils_1.isIndexOutOfBoundaries)(nextColIndex, 0, apiRef.current.getColumnCount() - 1)) {
                return;
            }
            //Is navigable?
            const col = apiRef.current.getColumnAt(nextColIndex);
            if (!col) {
                return logger.error(`Column not found at${nextColIndex}`);
            }
            if (col.disableNavigation) {
                nextColIndex = findNextNavigableColumnIndex(coords.colIndex, 'right');
            }
            if (apiRef.current.isMerged({ rowIndex: coords.rowIndex, colIndex: nextColIndex })) {
                const parentCoords = apiRef.current.getMergeParentCoords({
                    rowIndex: coords.rowIndex,
                    colIndex: nextColIndex,
                });
                if (parentCoords) {
                    return selectCell(parentCoords);
                }
                return logger.warn(`[Navigation] Merge group path not correct, returned ${parentCoords} positions instead of the expected coordinates. Please review`);
            }
            return selectCell({ rowIndex: coords.rowIndex, colIndex: nextColIndex });
        }
        if (event.key === 'ArrowLeft' || (event.key === 'Tab' && event.shiftKey)) {
            event.preventDefault();
            let nextColIndex = coords.colIndex - 1;
            if ((0, utils_1.isIndexOutOfBoundaries)(nextColIndex, 0, apiRef.current.getColumnCount() - 1)) {
                return;
            }
            const nextColumn = apiRef.current.getColumnAt(nextColIndex);
            if (!nextColumn) {
                return logger.error(`Column not found at ${nextColIndex}`);
            }
            if (nextColumn.disableNavigation) {
                nextColIndex = findNextNavigableColumnIndex(coords.colIndex, 'left');
            }
            if (apiRef.current.isMerged({ rowIndex: coords.rowIndex, colIndex: nextColIndex })) {
                const parentCoords = apiRef.current.getMergeParentCoords({
                    rowIndex: coords.rowIndex,
                    colIndex: nextColIndex,
                });
                if (parentCoords) {
                    return selectCell(parentCoords);
                }
                return logger.warn(`[Navigation] Merge group path not correct, returned ${parentCoords} positions instead of the expected coordinates. Please review`);
            }
            return selectCell({ rowIndex: coords.rowIndex, colIndex: nextColIndex });
        }
    }, [apiRef, coords, findNextNavigableColumnIndex, logger, selectCell]);
    const handleSelectionHeaderControls = (0, react_1.useCallback)((event, row) => {
        //Enable only checkbox via enter
        if (event.key === 'Enter') {
            return apiRef.current.selectRow(row);
        }
    }, [apiRef]);
    const onKeyDown = (0, react_1.useCallback)((event) => {
        var _a, _b;
        setWasCliked(false);
        const editorState = apiRef.current.getEditorState();
        //Ensure we can proceed navigation under this core conditions
        if (!initialised || suppressControls || (editorState === null || editorState === void 0 ? void 0 : editorState.isPopup)) {
            return logger.info(`Navigation is suppressed for key ${event.key} with the following states: ${JSON.stringify({
                initialised,
                suppressControls,
                isPopup: editorState === null || editorState === void 0 ? void 0 : editorState.isPopup,
            })}`);
        }
        const isCtrlPressed = (event.ctrlKey || event.metaKey) && !event.altKey;
        //Common strategy - Must be on top level, its priority over normal navigation
        if (editorState && (event.key === 'Enter' || event.key === 'Escape')) {
            return handleEditorOpenControls(event);
        }
        const selector = (0, utils_1.createCellQuerySelector)(coords);
        const cellElement = (_b = (_a = apiRef.current.rootElementRef) === null || _a === void 0 ? void 0 : _a.current) === null || _b === void 0 ? void 0 : _b.querySelector(selector);
        if (!cellElement) {
            /**
             * @todo Review the solution of scroll up to the selected coords
             */
            /*const gridRef = apiRef.current.getGridRef()
            const scrollOfSelected = gridRef?.getOffsetForCell({
              /!** @todo Alignment must come from props **!/
              alignment: 'center',
              rowIndex: coords.rowIndex,
              columnIndex: coords.colIndex,
            })
            if (scrollOfSelected) {
              return gridRef?.scrollToPosition(scrollOfSelected)
            }*/
            return logger.debug((0, utils_1.createSelectorElementNotFoundWarning)(coords));
        }
        const column = apiRef.current.getColumnAt(coords.colIndex);
        if (!column) {
            return logger.warn('Column not found');
        }
        //Travel like excel rules for non-editing
        if (!editorState) {
            if (event.key === 'F2') {
                event.preventDefault();
                return apiRef.current.beginEditing({
                    coords,
                    targetElement: cellElement,
                });
            }
            if (event.key === 'Enter' && onCreateRow && column.id !== rowSelection_1.ROW_SELECTION_HEADER_ID) {
                event.preventDefault();
                onCreateRow(coords);
            }
        }
        //Check if its a navigation key
        if (editorManager_1.NavigationKey[event.key] !== undefined && !editorState) {
            return handleArrowNavigationControls(event);
        }
        const row = apiRef.current.getRowAt(coords.rowIndex);
        if (!row) {
            return logger.warn(`Row not found at index ${coords.rowIndex}`);
        }
        const currentValue = row[column.accessor];
        //Handle specific keyboard handlers
        if (column.id === rowSelection_1.ROW_SELECTION_HEADER_ID) {
            return handleSelectionHeaderControls(event, row);
        }
        if (isCtrlPressed && !editorState) {
            return handleControlOrMetaPressedControls(event, column, row, currentValue);
        }
        //Handle cell deleting
        if ((event.key === 'Backspace' || event.key === 'Delete') && !editorState) {
            //If a dynamic callback/value is passed we resolve and in case its true we just exit
            if (column.disableBackspace !== undefined &&
                (0, resolveDynamicCallback_1.resolveDynamicCallback)(column.disableBackspace, { row, column })) {
                return;
            }
            event.preventDefault();
            const newValue = (0, utils_1.getDefaultValueFromValue)(currentValue);
            if (currentValue === newValue) {
                return;
            }
            return onCellChange === null || onCellChange === void 0 ? void 0 : onCellChange({
                newValue,
                previousValue: currentValue,
                coords,
                row,
                column,
            }, event.key === 'Backspace' ? 'backspace' : 'delete');
        }
        //If its numeric key and we are in numeric column, open with this key by default
        if (column.type === columnGrid_1.ColumnCellType.Numeric && !editorState) {
            const regex = /^[0-9]+$/;
            if (regex.test(event.key)) {
                event.preventDefault();
                return apiRef.current.beginEditing({
                    coords,
                    defaultKey: event.key !== 'Dead' ? event.key : undefined,
                    targetElement: cellElement,
                });
            }
        }
        //If its any printable char, we allow to open editing
        if (!column.type &&
            (0, utils_1.isPrintableChar)(event.keyCode) &&
            !(0, utils_1.isMetaKey)(event.keyCode) &&
            !editorState) {
            //Any key makes it to open and send the key pressed
            event.preventDefault();
            return apiRef.current.beginEditing({
                coords,
                defaultKey: event.key !== 'Dead' ? event.key : undefined,
                targetElement: cellElement,
            });
        }
    }, [
        apiRef,
        initialised,
        suppressControls,
        coords,
        handleEditorOpenControls,
        logger,
        onCreateRow,
        handleArrowNavigationControls,
        handleSelectionHeaderControls,
        handleControlOrMetaPressedControls,
        onCellChange,
    ]);
    const onCellClick = (0, react_1.useCallback)(({ event, colIndex, rowIndex, element }) => {
        event.preventDefault();
        setWasCliked(true);
        selectCell({ rowIndex, colIndex }, false, element);
    }, [selectCell]);
    const onCellDoubleClick = (0, react_1.useCallback)(({ event, colIndex, rowIndex, element }) => {
        event.preventDefault();
        setWasCliked(true);
        //Compare if the cell is equal to whats selected otherwise select it first
        if (colIndex !== coords.colIndex && rowIndex !== coords.rowIndex) {
            selectCell({ rowIndex, colIndex }, false, element);
        }
        apiRef.current.beginEditing({
            coords: { colIndex, rowIndex },
            targetElement: element,
        });
    }, [apiRef, selectCell, coords]);
    const getSelectedCoords = (0, react_1.useCallback)(() => coordsRef.current, []);
    (0, api_1.useApiEventHandler)(apiRef, 'keydown', onKeyDown);
    (0, api_1.useApiEventHandler)(apiRef, 'CELL_CLICK', onCellClick);
    (0, api_1.useApiEventHandler)(apiRef, 'CELL_DOUBLE_CLICK', onCellDoubleClick);
    const navigationApi = {
        getSelectedCoords,
        selectCell,
    };
    (0, api_1.useApiExtends)(apiRef, navigationApi, 'NavigationApi');
    return { coords, wasClicked };
}
exports.useKeyboard = useKeyboard;
