"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useMergeCells = void 0;
const react_1 = require("react");
const validateMergeData_1 = require("./validateMergeData");
const createMergedPositions_1 = require("./createMergedPositions");
const createMergedGroups_1 = require("./createMergedGroups");
const api_1 = require("../api");
const logger_1 = require("../logger");
/**
 * Provides validation for merged data, handles the lookup with navigation coordinates and
 * return util functions
 * @param data
 */
function useMergeCells({ mergeCells, rowCount, columnCount, apiRef }) {
    const logger = (0, logger_1.useLogger)('useMergeCells');
    const mergedPositions = (0, react_1.useRef)([]);
    const mergeGroups = (0, react_1.useRef)({});
    const mergeData = (0, react_1.useRef)([]);
    (0, react_1.useEffect)(() => {
        if (!mergeCells) {
            return;
        }
        logger.debug('Creating and validating merged cells');
        mergeData.current = (0, validateMergeData_1.validateMergeData)(mergeCells, rowCount, columnCount);
        mergeGroups.current = (0, createMergedGroups_1.createMergedGroups)(mergeCells);
        mergedPositions.current = (0, createMergedPositions_1.createMergedPositions)(mergeCells);
    }, [mergeCells, rowCount, columnCount, logger]);
    const isMerged = (0, react_1.useCallback)(({ rowIndex, colIndex }) => {
        return mergedPositions.current.some(e => e.row === rowIndex && e.col === colIndex);
    }, []);
    /**
     * Returns the parent of the merged position given
     * @param coords
     * @returns The parent coordinates or nothing
     */
    const getMergeParentCoords = (0, react_1.useCallback)(({ rowIndex, colIndex }) => {
        const targets = mergeData.current.filter(e => e.colIndex === colIndex);
        if (targets.length === 0) {
            console.error(`Please review your setup, target merge column not found at getMergedPath from navigation. Given input [${rowIndex}, ${colIndex}]`);
            return undefined;
        }
        //Search for the target that is between an interval of indices
        const target = targets.find(target => {
            const rowStart = target.rowIndex;
            const rowEnd = target.rowIndex + target.rowSpan - 1;
            return rowIndex >= rowStart && rowIndex <= rowEnd;
        });
        if (target) {
            return { rowIndex: target.rowIndex, colIndex: target.colIndex };
        }
        return undefined;
    }, []);
    /**
     * Returns the col/row span of the given colIndex/rowIndex
     * @param coords
     */
    const getSpanProperties = (0, react_1.useCallback)((coords) => mergeData.current.find(e => e.rowIndex === coords.rowIndex && e.colIndex === coords.colIndex), []);
    const getMergedData = (0, react_1.useCallback)(() => mergeData.current, []);
    const getMergedGroups = (0, react_1.useCallback)(() => mergeGroups.current, []);
    const mergedCellsApi = {
        getSpanProperties,
        isMerged,
        getMergeParentCoords,
        getMergedData,
        getMergedGroups,
    };
    (0, api_1.useApiExtends)(apiRef, mergedCellsApi, 'MergeCellsAPI');
    return { mergedPositions: mergedPositions.current, mergedCells: mergeData.current, isMerged };
}
exports.useMergeCells = useMergeCells;
