"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useNestedColumns = void 0;
const api_1 = require("../api");
const react_1 = require("react");
const logger_1 = require("../logger");
const createDepthMap_1 = require("./createDepthMap");
const lodash_1 = require("lodash");
/**
 * This hook exposes utilities and methods to interact with Columns children
 */
function useNestedColumns({ apiRef, enabled, initialised, defaultExpandedColumnsIds, }) {
    var _a;
    const expandedColumnIds = (0, react_1.useRef)([]);
    const defaultIdsRef = (0, react_1.useRef)((_a = (0, lodash_1.uniq)(defaultExpandedColumnsIds)) !== null && _a !== void 0 ? _a : []);
    const isDefaultIdsExpanded = (0, react_1.useRef)(false);
    const columnsDepthMap = (0, react_1.useRef)({});
    const logger = (0, logger_1.useLogger)('useNestedColumns');
    //When Columns changes we need to create the depth map
    //TODO: Might be a room for improvement just creating the depth map when the original Columns (Columns that come from props) change
    const onColumnsChanged = (0, react_1.useCallback)(({ columns }) => {
        columnsDepthMap.current = (0, createDepthMap_1.createDepthMap)(apiRef.current.getOriginalColumns(), apiRef.current.selectionKey);
        //Check if expandedColumnIds are still present on the new Columns
        const removedIds = [];
        expandedColumnIds.current.forEach(id => {
            const exists = columns.some(e => e[apiRef.current.selectionKey] === id);
            if (!exists) {
                logger.info(`Column id: ${id} does not exist therefore we need to remove it.`);
                removedIds.push(id);
            }
        });
        if (removedIds.length > 0) {
            logger.info(`Removing ${removedIds.length} ids that does not exist anymore after Columns change`);
            expandedColumnIds.current = expandedColumnIds.current.filter(e => !removedIds.some(i => e === i));
            const newColumns = columns.filter(e => !removedIds.some(id => id === e[apiRef.current.selectionKey]));
            apiRef.current.updateColumns(newColumns);
        }
    }, [apiRef, logger]);
    const toggleColumnExpand = (0, react_1.useCallback)((target) => {
        if (!enabled) {
            return logger.warn('nestedColumns are not enabled in order to perform a Column expand/collapse');
        }
        const ids = Array.isArray(target) ? target : [target];
        logger.info(`Toggling ${ids.length} columns`);
        ids.forEach(id => {
            if (expandedColumnIds.current.includes(id)) {
                expandedColumnIds.current = [...expandedColumnIds.current].filter(e => e !== id);
            }
            else {
                expandedColumnIds.current = [...expandedColumnIds.current, id];
            }
        });
        apiRef.current.dispatchEvent(api_1.COLLAPSES_COLUMNS_CHANGED, expandedColumnIds.current);
    }, [apiRef, enabled, logger]);
    const getExpandedColumns = (0, react_1.useCallback)(() => {
        return expandedColumnIds.current;
    }, []);
    const isColumnExpanded = (0, react_1.useCallback)((id) => {
        return expandedColumnIds.current.includes(id);
    }, []);
    const getColumnDepth = (0, react_1.useCallback)((id) => {
        var _a;
        return (_a = columnsDepthMap.current[id]) !== null && _a !== void 0 ? _a : 1;
    }, []);
    const nestedColumnsApi = {
        toggleColumnExpand,
        isColumnExpanded,
        getExpandedColumns,
        getColumnDepth,
    };
    //Load default ids
    (0, react_1.useEffect)(() => {
        if (!isDefaultIdsExpanded.current &&
            defaultIdsRef.current.length > 0 &&
            initialised &&
            enabled) {
            logger.info(`Expanded default ids: ${defaultIdsRef.current.length} on init state ${initialised}`);
            isDefaultIdsExpanded.current = true;
            toggleColumnExpand(defaultIdsRef.current);
        }
    }, [initialised, apiRef, toggleColumnExpand, logger, enabled]);
    (0, api_1.useApiEventHandler)(apiRef, api_1.COLUMNS_CHANGED, onColumnsChanged);
    (0, api_1.useApiExtends)(apiRef, nestedColumnsApi, 'NestedColumnsApi');
    return { expandedColumnIndices: expandedColumnIds.current, isColumnExpanded };
}
exports.useNestedColumns = useNestedColumns;
