"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useNestedRows = void 0;
const api_1 = require("../api");
const react_1 = require("react");
const logger_1 = require("../logger");
const createDepthMap_1 = require("./createDepthMap");
const lodash_1 = require("lodash");
/**
 * This hook exposes utilities and methods to interact with rows children
 */
function useNestedRows({ apiRef, enabled, initialised, defaultExpandedIds }) {
    var _a;
    const expandedRowIds = (0, react_1.useRef)([]);
    const defaultIdsRef = (0, react_1.useRef)((_a = (0, lodash_1.uniq)(defaultExpandedIds)) !== null && _a !== void 0 ? _a : []);
    const isDefaultIdsExpanded = (0, react_1.useRef)(false);
    const rowsDepthMap = (0, react_1.useRef)({});
    const logger = (0, logger_1.useLogger)('useNestedRows');
    //When rows changes we need to create the depth map
    //TODO: Might be a room for improvement just creating the depth map when the original rows (rows that come from props) change
    const onRowsChanged = (0, react_1.useCallback)(({ rows }) => {
        rowsDepthMap.current = (0, createDepthMap_1.createDepthMap)(apiRef.current.getOriginalRows(), apiRef.current.selectionKey);
        //Check if expandedRowIds are still present on the new rows
        const removedIds = [];
        expandedRowIds.current.forEach(id => {
            const exists = rows.some(e => e[apiRef.current.selectionKey] === id);
            if (!exists) {
                logger.info(`Row id: ${id} does not exist therefore we need to remove it.`);
                removedIds.push(id);
            }
        });
        if (removedIds.length > 0) {
            logger.info(`Removing ${removedIds.length} ids that does not exist anymore after rows change`);
            expandedRowIds.current = expandedRowIds.current.filter(e => !removedIds.some(i => e === i));
            const newRows = rows.filter(e => !removedIds.some(id => id === e[apiRef.current.selectionKey]));
            apiRef.current.updateRows(newRows);
        }
    }, [apiRef, logger]);
    const toggleRowExpand = (0, react_1.useCallback)((target) => {
        if (!enabled) {
            return logger.warn('nestedRows are not enabled in order to perform a row expand/collapse');
        }
        const ids = Array.isArray(target) ? target : [target];
        logger.info(`Toggling ${ids.length} rows`);
        ids.forEach(id => {
            if (expandedRowIds.current.includes(id)) {
                expandedRowIds.current = [...expandedRowIds.current].filter(e => e !== id);
            }
            else {
                expandedRowIds.current = [...expandedRowIds.current, id];
            }
        });
        apiRef.current.dispatchEvent(api_1.COLLAPSES_CHANGED, expandedRowIds.current);
    }, [apiRef, enabled, logger]);
    const getExpandedRows = (0, react_1.useCallback)(() => {
        return expandedRowIds.current;
    }, []);
    const isRowExpanded = (0, react_1.useCallback)((id) => {
        return expandedRowIds.current.includes(id);
    }, []);
    const getRowDepth = (0, react_1.useCallback)((id) => {
        var _a;
        return (_a = rowsDepthMap.current[id]) !== null && _a !== void 0 ? _a : 1;
    }, []);
    const nestedRowsApi = {
        toggleRowExpand,
        isRowExpanded,
        getExpandedRows,
        getRowDepth,
    };
    //Load default ids
    (0, react_1.useEffect)(() => {
        if (!isDefaultIdsExpanded.current &&
            defaultIdsRef.current.length > 0 &&
            initialised &&
            enabled) {
            logger.info(`Expanded default ids: ${defaultIdsRef.current.length} on init state ${initialised}`);
            isDefaultIdsExpanded.current = true;
            toggleRowExpand(defaultIdsRef.current);
        }
    }, [initialised, apiRef, toggleRowExpand, logger, enabled]);
    (0, api_1.useApiEventHandler)(apiRef, api_1.ROWS_CHANGED, onRowsChanged);
    (0, api_1.useApiExtends)(apiRef, nestedRowsApi, 'NestedRowsApi');
    return { expandedRowIndices: expandedRowIds.current, isRowExpanded };
}
exports.useNestedRows = useNestedRows;
