"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useRowSelection = exports.ROW_SELECTION_HEADER_ID = void 0;
const react_1 = require("react");
const api_1 = require("../api");
const logger_1 = require("../logger");
exports.ROW_SELECTION_HEADER_ID = '__selection__';
/**
 * useRowSelection is a hook that provides utility for selecting rows
 * @param apiRef
 * @param initialised
 * @param selection
 */
function useRowSelection(apiRef, initialised, selection) {
    const logger = (0, logger_1.useLogger)('useRowSelection');
    const selectedIds = (0, react_1.useRef)([]);
    const selectionRef = (0, react_1.useRef)(selection);
    (0, react_1.useEffect)(() => {
        selectionRef.current = selection;
    }, [selection]);
    //Detect if a row exists in selected but not in rows
    (0, react_1.useEffect)(() => {
        if (!selection) {
            logger.debug('Removing selection ids.');
            //Ensure cleanup because selection might have been disabled
            selectedIds.current = [];
            apiRef.current.dispatchEvent(api_1.ROW_SELECTION_CHANGE);
        }
    }, [apiRef, selection, selectedIds, logger]);
    const isRowSelected = (0, react_1.useCallback)((id) => selectedIds.current.some(e => e === id), []);
    const selectRow = (0, react_1.useCallback)((idOrRow) => {
        logger.debug(`Selecting row ${idOrRow.toString()}`);
        //Ensure selection is enabled
        if (!selectionRef.current) {
            return logger.info('Selection is disabled.');
        }
        const _id = typeof idOrRow !== 'object' ? idOrRow : idOrRow[selectionRef.current.key];
        //Find the target row in order to determinate whether we can select or not
        const targetRow = apiRef.current.getRowById(String(_id));
        if (!targetRow) {
            logger.info({
                rows: apiRef.current.getRows(),
                id: _id,
                selectionKey: selectionRef.current.key,
            });
            return logger.warn(`Row not found (result= ${targetRow}) with the given key ${selectionRef.current.key} 
          on param: ${idOrRow} and extracted the id: ${_id} with total rows ${apiRef.current.getRowsCount()}`);
        }
        //If we do have the middleware and it returns false, just block
        if (selectionRef.current.canSelect && !selectionRef.current.canSelect(targetRow)) {
            return logger.info(`Row id: ${_id} cannot be selected`);
        }
        //Check if highlight is at the selecting id otherwise we need to force it
        const rowIndex = apiRef.current.getRowIndex(String(_id));
        apiRef.current.selectCell({
            colIndex: apiRef.current.getColumnCount() - 1,
            rowIndex,
        });
        //Toggle effect
        if (!isRowSelected(_id)) {
            selectedIds.current = [...selectedIds.current, _id];
        }
        else {
            selectedIds.current = [...selectedIds.current.filter(e => e !== _id)];
        }
        apiRef.current.dispatchEvent(api_1.ROW_SELECTION_CHANGE);
    }, [logger, apiRef, isRowSelected]);
    const getSelectedRows = (0, react_1.useCallback)(() => {
        if (!selectionRef.current) {
            return [];
        }
        return apiRef.current
            .getRowsWithFilter(e => selectedIds.current.some(id => String(id) === String(e[selectionRef.current.key])))
            .map((e) => String(e[selectionRef.current.key]));
    }, [apiRef]);
    const rowSelectionApi = {
        isRowSelected,
        selectRow,
        getSelectedRowIds: getSelectedRows,
    };
    (0, api_1.useApiExtends)(apiRef, rowSelectionApi, 'RowSelectionApi');
    return {
        isRowSelected,
        selectRow,
        getSelectedRows,
    };
}
exports.useRowSelection = useRowSelection;
